import * as util from 'util';

// noinspection JSUnusedGlobalSymbols
class Logger {
  static options: util.InspectOptions = {};
  protected static instance: Logger = new Logger();

  protected constructor() {
  }

  // noinspection JSUnusedGlobalSymbols
  static dir(item: any, options: util.InspectOptions = Logger.options) {
    const text = Logger.inspectWrapper(item, options);
    return console.log(text);
  }

  // noinspection FunctionWithMultipleReturnPointsJS
  static inspectName(item: object) {
    const custom = util.inspect.custom;

    if (null === item) {
      return null;
    }

    if (undefined !== (<any>item)[custom]) {
      return custom;
    }

    if (undefined !== (<any>item)['inspect']) {
      return 'inspect';
    }

    return null;
  }

  static inspectWrapper(item: any, options: util.InspectOptions = Logger.options) {
    const logger = Logger.instance;
    const oldInspect = logger.bindInspect(item, logger.inspect);
    const text = util.inspect(item, options);
    logger.bindInspect(item, oldInspect);
    return text;
  }

  // noinspection FunctionWithMultipleReturnPointsJS
  bindInspect(item: any, inspect: any = this.inspect) {
    if (('object' !== typeof item) || (null === item)) {
      return null;
    }

    const name = Logger.inspectName(item);

    if ((null !== inspect) && (null === name)) {
      item.inspect = inspect.bind(item);
    }

    if ((null === inspect) && (null !== name)) {
      delete item[name];
    }

    return name;
  }

  // noinspection FunctionWithMultipleReturnPointsJS
  inspect(depth: number | null, options: any) {
    if ((null !== depth) && (0 > depth)) {
      return options.stylize('<MaxDepth>', 'special');
    }

    const newDepth = (null === options.depth) ? null : options.depth - 1;
    const newOptions = Object.assign({}, options, {depth: newDepth});
    const name = Logger.inspectName(this);
    const isArray = this instanceof Array;
    const replace = '\n  ';
    const items = [];

    for (let index in this) {
      const item = this[index];

      if ((name === index) && ('function' === typeof item)) {
        // noinspection ContinueStatementJS
        continue;
      }

      const text = Logger.inspectWrapper(item, newOptions).replace(/\n/g, replace);
      items.push(`${index}: ${text}`);
    }

    const text = items.join(`,${replace}`);

    if (isArray) {
      return ('' === text) ? '[]' : `[${replace}${text}\n]`;
    } else {
      return ('' === text) ? '{}' : `{${replace}${text}\n}`;
    }
  }

}

export default Logger;
